# Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.

# Disable build id note requirement for now
%undefine _missing_build_ids_terminate_build

# Force special prefix for Vespa
%define _prefix /opt/vespa-deps

%global debug_package %{nil}

# Don't provide shared library or pkgconfig
%global __provides_exclude ^(lib.*\\.so[0-9.]*\\(\\)\\(64bit\\)|(cmake|pkgconfig)\\(.*)$

%global __requires_exclude ^lib(jllama|llama|ggml)\.so[0-9.]*\\([A-Z._0-9]*\\)\\(64bit\\)$

# Version
%define ver_major 4
%define ver_minor 1
%define ver_patch 0
%define ver_release 4

Summary:        Native part of Java Bindings for llama.cpp using CUDA
Name:           vespa-jllama-cuda
Version:        %{ver_major}.%{ver_minor}.%{ver_patch}
Release:        %{ver_release}%{?dist}
License:        MIT
URL:            https://github.com/kherud/
Source0:        https://github.com/kherud/java-llama.cpp/archive/refs/tags/v%{version}.tar.gz

%if 0%{?el8} || 0%{?el9} || 0%{?fedora}
BuildRequires: java-17-openjdk-devel
BuildRequires: maven
%endif

%if 0%{?el8}%{?el9}
BuildRequires: vespa-toolset-14-meta
%define _devtoolset_enable /opt/rh/gcc-toolset-14/enable
BuildRequires: vespa-ccache
BuildRequires: vespa-cmake
%else
BuildRequires: cmake
%endif
BuildRequires: make

%define _cuda_version_major 12
%define _cuda_version_minor 8
%define _cuda_version_micro 90
%define _cuda_version_pkg_name %{_cuda_version_major}-%{_cuda_version_minor}
%define _cuda_version_norm_num %{_cuda_version_major}.%{_cuda_version_minor}
%define _cuda_version_full_num %{_cuda_version_major}.%{_cuda_version_minor}.%{_cuda_version_micro}
BuildRequires: cuda-toolkit-12-config-common = %{_cuda_version_full_num}
BuildRequires: cuda-toolkit-config-common = %{_cuda_version_full_num}
BuildRequires: cuda-libraries-devel-%{_cuda_version_pkg_name}
BuildRequires: cuda-compiler-%{_cuda_version_pkg_name}
BuildRequires: cuda-cudart-devel-%{_cuda_version_pkg_name}
BuildRequires: cuda-command-line-tools-%{_cuda_version_pkg_name}

Requires:       cuda-cudart-%{_cuda_version_name}
Requires:       libcublas-%{_cuda_version_name}

%global _vespa_3rdparty_deps_packaging_notice \
See https://github.com/vespa-engine/vespa-3rdparty-deps for details \
about packaging.

%description
%{_vespa_3rdparty_deps_packaging_notice}

%prep
%setup -q -n java-llama.cpp-%{version}

%build
%if 0%{?_devtoolset_enable:1}
source %{_devtoolset_enable} || true
%endif

PATH=%{_prefix}/bin:/usr/local/cuda-%{_cuda_version_norm_num}/bin:$PATH
mvn compile
mkdir build

cmake -S . -B build \
    -DCMAKE_INSTALL_RPATH=\$ORIGIN \
    -DCMAKE_BUILD_WITH_INSTALL_RPATH=true \
    -DCMAKE_BUILD_TYPE=RelWithDebInfo \
    -DCMAKE_C_FLAGS_RELWITHDEBINFO="-g -O3 -DNDEBUG" \
    -DCMAKE_CXX_FLAGS_RELWITHDEBINFO="-g -O3 -DNDEBUG" \
    -DGGML_METAL=OFF \
    -DGGML_OPENMP=ON \
    -DGGML_NATIVE=OFF \
    -DGGML_CUDA=ON -DCMAKE_CUDA_ARCHITECTURES="75;80;89" \
    -DLLAMA_NATIVE=OFF

VERBOSE=1 cmake --build build --config RelWithDebInfo -j 4

%install

mkdir -p %{buildroot}%{_libdir}/cuda
cp -p src/main/resources_linux_cuda/de/kherud/llama/Linux/*/lib*.so %{buildroot}%{_libdir}/cuda

%files
%license LICENSE.md
%{_libdir}

%changelog
